/***************************************************************************************
 * @file     I2C_AN_main.c
 * @version  V1.0.9
 * @date     3/12/2019  CEM 
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "reb_adt75.h"
#include "driver_common.h"
#include "reb_max11619.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"
#include "segger_rtt.h" 
#include "irq_va108xx.h"

volatile uint8_t bounce_cnt = 0; 

//**************************************************************
//  Global variables declared here  
//
//**************************************************************

volatile uint32_t ADC_cnt,  ADC_results[5] ;   //  used inside ISR and in main routine  
 

//**** global variable defs

#define PERIPH_CLK_ENAB_PORTA 0x00 
#define PERIPH_CLK_ENAB_PORTB 0x01
#define PERIPH_CLK_ENAB_SPIA 0x04
#define PERIPH_CLK_ENAB_SPIB 0x05
#define PERIPH_CLK_ENAB_SPIC 0x06
#define PERIPH_CLK_ENAB_UARTA 0x8
#define PERIPH_CLK_ENAB_UARTB 0x9
#define PERIPH_CLK_ENAB_I2CA 0x10
#define PERIPH_CLK_ENAB_I2CB 0x11
#define PERIPH_CLK_ENAB_IRQSEL 0x15
#define PERIPH_CLK_ENAB_IOCONFIG 0x16
#define PERIPH_CLK_ENAB_UTILITY 0x17
#define PERIPH_CLK_ENAB_PORTIO 0x18 
#define FUNCSEL0 00
#define FUNCSEL1 01 
#define FUNCSEL2 02 
		//  *** Cypress FM25V10 (SPI FRAM) device command set: 40 MHz access  ****		
#define FRAM_WREN 0x06
#define FRAM_WRDI 0x04
#define FRAM_RDSR 0x05
#define FRAM_WRSR 0x01
#define FRAM_READ 0x03
#define FRAM_WRITE 0x02
#define FRAM_RDID 0x9F 
		
		// Address masks and macros for creating byte by byte address declarations
#define FADDR_MSB_MASK   (uint32_t)0xFF0000
#define FADDR_MID_MASK   (uint32_t)0x00FF00
#define FADDR_LSB_MASK   (uint32_t)0x0000FF
#define FMSB_ADDR_BYTE(addr)   ((uint8_t)((addr & FADDR_MSB_MASK)>>16))
#define FMID_ADDR_BYTE(addr)   ((uint8_t)((addr & FADDR_MID_MASK)>>8))
#define FLSB_ADDR_BYTE(addr)   ((uint8_t)(addr & FADDR_LSB_MASK))



#define SPI_PERIPHERAL_CTRL1_BLOCKMODE_Pos     7                                                       /*!< SPI_PERIPHERAL CTRL1: BYTEMODE Position */
#define SPI_PERIPHERAL_CTRL1_BLOCKMODE_Msk     (0x01UL << SPI_PERIPHERAL_CTRL1_BLOCKMODE_Pos)
   
  // *********************

uint32_t  __attribute__((aligned(32))) READ_APB(volatile const uint32_t *ptr) {
	 return *ptr;
}

void  __attribute__((aligned(32))) WRITE_APB(volatile uint32_t *ptr, uint32_t data) {
	 *ptr=data;
}









//************ init_i2cb *********
//  Steps: 
//     0: enable clocks to I2Cb
//     1: load CLKSCALE to set bit rate  
//     2:  Clear Rx and Tx FIFO
//     3:  Load CTRL with parameters (dig filter enabled & enable bit) 

uint32_t init_i2cb() 
{
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CB  ;  
	
	VOR_I2CB -> CLKSCALE = 0x0018  ;  // default to normal mode, divide by 25 for 50 MHz SysClk
	VOR_I2CB -> FIFO_CLR = 0x3 ;  // clear both Rx and Tx FIFO  
	VOR_I2CB -> CTRL =  (I2CB_CTRL_DLGFILTER_Msk | I2CB_CTRL_ENABLE_Msk)  ;   // enable filter and module 
	return(VOR_I2CB -> STATUS)  ; 
	
}



	//  ****** Write to DAC on I2CB     *********     
	
		// 
		//   Basic flow is this:
		//      1) init clocks for peripherals
		//			2) init SPI control registers (0 & 1)  
		//      3) send Write enable command (single byte)  {wait for transmit complete} 
		//      4) wait for frame to complete

uint32_t write_DAC_I2CB ( uint32_t I2C_Address, uint32_t DAC_out ) 
{
volatile		 int32_t count = 0, count2=0, result, temp0, temp1, temp_ctrl, xtemp, x  ; 
	
	VOR_I2CB -> WORDS = 3  ;   // DAC has address + two bytes of data
	VOR_I2CB -> ADDRESS = I2C_Address & ~ 0x01  ;  
	VOR_I2CB -> DATA = 0x0 ;   // Address = 0x0 for writing data
	VOR_I2CB -> DATA = (DAC_out >> 2) & ~0xFFFFF00  ;  //  write first 8 bits of DAC_out (upper 8 bits of 10)
	VOR_I2CB -> DATA  = (DAC_out << 6) & ~0xFFFFF3F  ;  // write 2nd byte of output data (lower 2 bits in pos 7 & 6)
	VOR_I2CB -> CMD = 0x03  ;  // start with stop 
	
	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) 
	{
		count++ ;  
		if (count > 0x100000) break   ; 
	}
	return(VOR_I2CB -> STATUS)  ; 
}


	//  ****** Read ADC on I2CB     *********
// Steps:
// 1)  Load WORDS with size of transaction 
// 2) Load I2C ADDRESS of ADC 
// 3) Load required DATA in to DATA FIFO 
//  4) Load CMD register with 0x03 (start with stop) 
//  5) Poll STATUS until transaction is complete
//  6) Process Rx_FIFO, shift and concatentate data for 10-bit result 

uint32_t read_adc_i2cb ( uint32_t I2C_Address )

{
	volatile		 int32_t count = 0, count2=0, result, temp2 ;  
	
	VOR_I2CB -> WORDS = 2  ;   // ADC has two byte result
	VOR_I2CB -> ADDRESS = I2C_Address & ~ 0x01  ;  
	VOR_I2CB -> CMD = 0x03  ;  // start with stop 
	
	VOR_I2CB -> DATA = 0x0 ;   // Address = 0x0 for writing data

	VOR_I2CB -> CMD = 0x03  ;  // start with stop 
	
	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) 
	{
		count++ ;  
		if (count > 0x100000) break   ; 
	}


  result = VOR_I2CB -> DATA ; 
  result = (result << 0x08) | VOR_I2CB -> DATA ;  // shift over first byte, read second byte and combine the two
	
	return(result)  ; 
} 
		
	

	//  ****** Read Temperature Sensor I2CB     *********
// Steps:
//  1) Write address to Read 
// 		1a)  Load WORDS with size of transaction =1 byte address  
// 		1b)) Load I2C ADDRESS of Temp sensor = 0x92 
//    1c) Load required DATA in to DATA FIFO (address = 0x0  
//    1d) Load CMD register with 0x03 (start with stop) 
//    1e) Poll STATUS until transaction is complete
//  2) Read temp sensor information 
// 		2a) Load WORDS with size of transaction =2 byte address  
// 		2b) Load I2C ADDRESS of Temp sensor = 0x92 
//    2c) Load CMD register with 0x03 (start with stop) 
//    2e) Poll STATUS until transaction is complete
//    2f) Read RxFIFO and align result

uint32_t read_temp_i2cb ( uint32_t I2C_Address )

{
	volatile		 int32_t count = 0, count2=0, result, temp0 ;  
	
	VOR_I2CB -> WORDS = 1  ;   // Temp sensor has one byte result
	VOR_I2CB -> ADDRESS = I2C_Address & ~ 0x01  ;   // set lsb = 0 (write)  
	VOR_I2CB -> CMD = 0x03  ;  // start with stop 
	
	
	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) // wait for transaction to complete
	{
		count++ ;  
		if (count > 0x100000) break   ; 
	}

	VOR_I2CB -> WORDS = 2  ;   // Temp sensor has one byte result
	VOR_I2CB -> ADDRESS = I2C_Address | 0x01  ;  // set slave address with lsb =1 (read) 
	VOR_I2CB -> CMD = 0x03  ;  // start with stop 

		while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) // wait for transaction to complete
	{
		count++ ;  
		if (count > 0x100000) break   ; 
	}
  temp0 = VOR_I2CB -> DATA ; 
  result = (temp0 << 0x08) | VOR_I2CB -> DATA ;  // shift over first byte, read second byte and combine the two
	
	return(result)  ; 
} 

//*************************************************************************
	

//  ****** Initialize I2CB for interrupts on IRQ22     *********
// Steps:
//  1) Configure I2CB
//  2) Configure IRQ_SEL peripheral 
//  3) Setup NVIC and enable interrupts on IRQ22 
// 	

#define I2CB_INTn  22 
#define I2CB_Priority  1  


int32_t init_i2cb_IRQ22()  
{

	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_I2CB  ;  
	
	VOR_I2CB -> CLKSCALE = 0x0018  ;  // default to normal mode, divide by 25 for 50 MHz SysClk
	VOR_I2CB -> FIFO_CLR = 0x3 ;  // clear both Rx and Tx FIFO  
	VOR_I2CB -> CTRL =  (I2CB_CTRL_DLGFILTER_Msk | I2CB_CTRL_ENABLE_Msk)  ;   // enable filter and module 
  VOR_I2CB -> IRQ_ENB = I2CB_STATUS_IDLE_Msk  ;  // generate interrupt when IDLE = 0 

   VOR_IRQSEL->I2C_MS[1] = I2CB_INTn   ;    

   NVIC_SetPriority(I2CB_INTn,I2CB_Priority)  ;  
   NVIC_EnableIRQ(I2CB_INTn)  ;  

return(VOR_I2CB -> STATUS  )  ;  	


}

//  ******* Interrupt Subroutine for I2CB  (OC22_IRQHandler)  
//  This will kick off a series of 4, 3-byte i2cb transactions to read a 10-bit ADC
//   After the 4th read, the I2CB will be idled and interrupts disabled 
//  The first time into the interrupt will be for the idle status prior to any i2c transactions 
// Entries 2-5 will be for idle condition post ADC conversion
// ADC_cnt is global variable which is set to zero before INT22 is enabled 
//  Individual ADC results are stored in global variables:  ADC_result[ADC_cnt]  

#define PORTB13  13   

void OC22_IRQHandler (void)  
{
	uint32_t result, count  ;  
	
	VOR_GPIO->BANK[1].DATAOUT |= (1<<PORTB13)  ;  
	
	if(ADC_cnt > 0)  // process data in RX FIFO for entries 2-5  (ADC_cnt:  1-4 )  
	{
		result = VOR_I2CB -> DATA  ;  
		result = (result << 0x08) | VOR_I2CB -> DATA  ;  // shift over first 
		ADC_results[ADC_cnt] = result  ;  
		
	
  } 
  
	if(ADC_cnt > 3)  // this was fourth sample so shut down I2CB)  
	{
		  VOR_I2CB -> IRQ_ENB = 0x00  ;  // generate interrupt when IDLE = 0 
		  ADC_results[0] = (ADC_results[1] + ADC_results[2] +ADC_results[3]+ ADC_results[4]) / 4  ; 
	}	 
  else
	{
  VOR_I2CB -> WORDS = 2  ;   // Temp sensor has one byte result
	VOR_I2CB -> ADDRESS = 0x9A | 0x01  ;  // set slave address with lsb =1 (read) 
	VOR_I2CB -> CMD = 0x03  ;  // start with stop     

	
	while(( VOR_I2CB -> STATUS & I2CB_STATUS_IDLE_Msk ) == 0) // wait for transaction to complete
	   {
		  count++ ;  
		  if (count > 0x100000) break   ; 
	   }
	}	

		ADC_cnt ++ ;  //  increment counter 	
	  VOR_GPIO->BANK[1].DATAOUT &= ~(1<<PORTB13)  ;  //  debug use only clear pin on ISR exit 
	
}

//**************************************************************


// *************************************************************
 
void Output_data_RTT(unsigned long adr, unsigned long sz, unsigned char *buf) 
{
	int32_t i=0, jj, kk ;  

   for(jj=0; jj<sz/32; jj++ ) 
		 {
 			  SEGGER_RTT_printf(0, "ADR %5X : ", adr  )  ;
			
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
				i+=4 ; 
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
				i+=4 ;  
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
				i+=4 ;  
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X  \r\n", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
			  i+=4 ;
				VOR_GPIO->BANK[0].TOGOUT |= (0x01UL << 10)   ; //drive 1 LED on 
			  adr += 16  ;  
			 	SEGGER_RTT_printf(0, "ADR %5X : ", adr  )  ;
			
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
				i+=4 ; 
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
				i+=4 ;  
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
				i+=4 ;  
				SEGGER_RTT_printf(0, " %2X %2X %2X %2X  \r\n", buf[i],buf[i+1],buf[i+2], buf[i+3] )  ;
				i+=4 ;
			  adr += 16  ;
			  for(kk=0;kk<0x8200;kk++)  {   }    //  wait a while to let buffer clear 
		 }
 }

//------------------------------------------------------------------------------
// Main Test Program
//  Setup LED and ADC  
//  continually convert ADC   
//------------------------------------------------------------------------------

uint32_t ctr = 0, tmp_RESET_STATUS, EE_STAT;
 
char abDataIn[128], abDataOut[128] ;   //  RTT (real time terminal) buffer allocation

#define SEGGER_RTT_IN_RAM 1
int main(void) 
{
		int32_t i =0, kk; 
		int32_t systick_old, elapsed_ticks1, elapsed_ticks2, main_cnt, retVal; 
		unsigned long adr, sz ;
	  unsigned char buf[256],*ibuf, *iread_buf, first_pass ; 
	
 	 VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= 
	       ((1<<PERIPH_CLK_ENAB_PORTIO) | (1<<PERIPH_CLK_ENAB_IOCONFIG) 
	        | (1<<PERIPH_CLK_ENAB_PORTA) | (1<<PERIPH_CLK_ENAB_PORTB)
	        | (1<<PERIPH_CLK_ENAB_IRQSEL) |(1<<PERIPH_CLK_ENAB_SPIA)); // enable clks to PORTIO, IOMGR and SPIB peripherals 
	
	for(i=0; i<32; i++  )   //  config all port pins with internal pull resistor enabled 
		{
	    VOR_IOCONFIG->PORTA[i] |= IOCONFIG_PORTA_PEN_Msk  ;
			if(i<24) 
			{ 
				VOR_IOCONFIG->PORTB[i] |= IOCONFIG_PORTB_PEN_Msk  ;
			}
		}
		VOR_GPIO->BANK[0].DIR |= ( (0x01UL << PORTA_6_D4) |(0x01UL << PORTA_7_D3) | (0x01UL << PORTA_10_D2) ) ; // setup REB1 LED pins  : PORTA 6,7,10
		VOR_GPIO->BANK[0].DATAOUT &= ~((0x01UL << PORTA_6_D4) |(0x01UL << PORTA_7_D3)  )  ; //drive  2 LEDs off
		VOR_GPIO->BANK[0].DATAOUT |= (0x01UL << PORTA_10_D2)   ; //drive 1 LED on 

		SysTick -> CTRL = (SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk) ; /*  enable Systick counter  */

		SysTick -> LOAD =  0x7fffff ; 
    systick_old = SysTick ->VAL ; 
		
		SEGGER_RTT_Init() ; //  Must call this for RAM based apps 
	  retVal = SEGGER_RTT_ConfigUpBuffer(1, "DataOut", &abDataOut, sizeof(abDataOut), SEGGER_RTT_MODE_BLOCK_IF_FIFO_FULL);
		retVal = SEGGER_RTT_ConfigDownBuffer(1, "DataIn", &abDataIn[0], sizeof(abDataIn), SEGGER_RTT_MODE_NO_BLOCK_SKIP);
 
		
		SEGGER_RTT_printf(0, "\n\n*****  AN1208 Example code terminal output  *****\n\n")  ; 	

		
// **  Example 1 - DAC write of AN1208 starts here 
		SEGGER_RTT_printf(0, "\n\n>>>>>  Example 1 - Read of FRAM Status Register\n")  ; 
		
		init_i2cb()  ;  
		write_DAC_I2CB(0x90,0x55)   ;  
    
    if(kk!= 0)
		{
      SEGGER_RTT_printf(0, "FRAM Status register = %5X \n\n", kk  )  ;
		}
//  **  Example 2 of Read ADC AN1208 starts here   **  
		SEGGER_RTT_printf(0, "\n\n>>>>>  Example 2 - 32 byte read from EEPROM (polling method) \n")  ; 
		 
		SEGGER_RTT_printf(0, "\n   +++ 32bytes from Boot EEPROM (polling method)  +++\n")  ; 
 	  
		init_i2cb()  ;  
		read_adc_i2cb(0x92)   ;  
		
 
				
// ** Example 3 - ADC read with Interrupts enabled of AN1208 starts here  **  
		SEGGER_RTT_printf(0, "\n\n>>>>> Example 3 - ADC read with interrupt enabled)\n")  ; 			
	  
		
		
		 init_i2cb_IRQ22() ;   // data from ADC read with Interrupt enabled 
		 
		SEGGER_RTT_printf(0, "\n+++  READ 32bytes from FRAM DATA  +++\n")  ;
		SEGGER_RTT_printf(0, "   Note: First three bytes account for READ command and 2 address  bytes. \n\n")  ; 
 
     first_pass = 1  ;  
		  
		 while(1)     //  This is infinite loop  (reads FRAM and EEPROM from Examples 2 & 3 continuously) 
		 {
	
		 VOR_GPIO->BANK[0].TOGOUT = (0x01UL << 6) ; // toggle LEDs each time through 

		 elapsed_ticks1 = systick_old - SysTick ->VAL   ;
		 systick_old = SysTick ->VAL ;   // time stamp of system tick counter		 
		
     
		 elapsed_ticks2 = systick_old - SysTick ->VAL   ;
			 

		
		 main_cnt ++ ; 
		}  //  end of while (1)   
		
				
	}  // end of main routine 
	
	
 
	





